@@ -3,7 +3,7 @@
  * buildargv.c
  *	  a routines for parsing string to argc, argv format
  *
- * Portions Copyright (c) 2017-2020 Pavel Stehule
+ * Portions Copyright (c) 2017-2021 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/buildargv.c
@@ -92,6 +92,13 @@ static struct option long_options[] =
 	{"wait", required_argument, 0, 36},
 	{"hold-stream", required_argument, 0, 37},
 	{"skip-columns-like", required_argument, 0, 38},
+	{"pgcli-fix", no_argument, 0, 39},
+	{"style", required_argument, 0, 's'},
+	{"reprint-on-exit", no_argument, 0, 'X'},
+	{"ascii", no_argument, 0, 'a'},
+	{"blackwhite", no_argument, 0, 'b'},
+	{"freezecols", required_argument, 0, 'c'},
+	{"no-xterm-mouse-mode", no_argument, 0, 40},
 	{0, 0, 0, 0}
 };
 
@@ -228,77 +235,79 @@ readargs(char **argv,
 		{
 			case 1:
 				{
-					fprintf(stderr, "pspg is a Unix pager designed for table browsing.\n\n");
-					fprintf(stderr, "Usage:\n");
-					fprintf(stderr, "  %s [OPTION] [file]\n", argv[0]);
-					fprintf(stderr, "\nGeneral options:\n");
-					fprintf(stderr, "  --about                  about authors\n");
-					fprintf(stderr, "  --help                   show this help\n");
-					fprintf(stderr, "  -V, --version            show version\n");
-					fprintf(stderr, "  -f, --file=FILE          open file\n");
-					fprintf(stderr, "  -F, --quit-if-one-screen\n");
-					fprintf(stderr, "                           quit if content is one screen\n");
-					fprintf(stderr, "  --hold-stream=NUM        can reopen closed FIFO (0, 1, 2)\n");
-					fprintf(stderr, "  --interactive            force interactive mode\n");
-					fprintf(stderr, "  --ignore_file_suffix     don't try to deduce format from file suffix\n");
-					fprintf(stderr, "  --ni                     not interactive mode (only for csv and query)\n");
-					fprintf(stderr, "  --no-watch-file          don't watch inotify event of file\n");
-					fprintf(stderr, "  --no-mouse               don't use own mouse handling\n");
-					fprintf(stderr, "  --no-sigint-search-reset\n");
-					fprintf(stderr, "                           without reset searching on sigint (CTRL C)\n");
-					fprintf(stderr, "  --only-for-tables        use std pager when content is not table\n");
-					fprintf(stderr, "  --on-sigint-exit         without exit on sigint(CTRL C or Escape)\n");
-					fprintf(stderr, "  --quit-on-f3             exit on F3 like mc viewers\n");
-					fprintf(stderr, "  --rr=ROWNUM              rows reserved for specific purposes\n");
-					fprintf(stderr, "  --stream                 read input forever\n");
-					fprintf(stderr, "  -X                       don't use alternate screen\n");
-					fprintf(stderr, "\nOutput format options:\n");
-					fprintf(stderr, "  -a                       force ascii\n");
-					fprintf(stderr, "  -b                       black-white style\n");
-					fprintf(stderr, "  -s N                     set color style number (0..%d)\n", MAX_STYLE);
-					fprintf(stderr, "  --bold-labels            row, column labels use bold font\n");
-					fprintf(stderr, "  --bold-cursor            cursor use bold font\n");
-					fprintf(stderr, "  --border                 type of borders (0..2)\n");
-					fprintf(stderr, "  --double-header          header separator uses double lines\n");
-					fprintf(stderr, "  --force-uniborder        replace ascii borders by unicode borders\n");
-					fprintf(stderr, "  --ignore-bad-rows        rows with wrong column numbers are ignored\n");
-					fprintf(stderr, "  --null=STRING            STRING used instead NULL\n");
-					fprintf(stderr, "\nSearching options\n");
-					fprintf(stderr, "  -g --hlite-search, -G --HILITE-SEARCH\n");
-					fprintf(stderr, "                           don't highlight lines for searches\n");
-					fprintf(stderr, "  -i --ignore-case         ignore case in searches that do not contain uppercase\n");
-					fprintf(stderr, "  -I --IGNORE-CASE         ignore case in all searches\n");
-					fprintf(stderr, "\nInterface options:\n");
-					fprintf(stderr, "  -c N                     fix N columns (0..9)\n");
-					fprintf(stderr, "  --less-status-bar        status bar like less pager\n");
-					fprintf(stderr, "  --line-numbers           show line number column\n");
-					fprintf(stderr, "  --no-bars, --no-commandbar, --no-topbar\n");
-					fprintf(stderr, "                           don't show bottom, top bar or both\n");
-					fprintf(stderr, "  --no-cursor              row cursor will be hidden\n");
-					fprintf(stderr, "  --no-sound               don't use beep when scroll is not possible\n");
-					fprintf(stderr, "  --tabular-cursor         cursor is visible only when data has table format\n");
-					fprintf(stderr, "  --vertical-cursor        show vertical column cursor\n");
-					fprintf(stderr, "\nInput format options:\n");
-					fprintf(stderr, "  --csv                    input stream has csv format\n");
-					fprintf(stderr, "  --csv-separator          char used as field separator\n");
-					fprintf(stderr, "  --csv-header [on/off]    specify header line usage\n");
-					fprintf(stderr, "  --skip-columns-like=\"SPACE SEPARATED STRING LIST\"\n");
-					fprintf(stderr, "                           columns with substr in name are ignored\n");
-					fprintf(stderr, "  --tsv                    input stream has tsv format\n");
-					fprintf(stderr, "\nWatch mode options:\n");
-					fprintf(stderr, "  -q, --query=QUERY        execute query\n");
-					fprintf(stderr, "  -w, --watch time         the query (or read file) is repeated every time (sec)\n");
-					fprintf(stderr, "\nConnection options\n");
-					fprintf(stderr, "  -d, --dbname=DBNAME      database name\n");
-					fprintf(stderr, "  -h, --host=HOSTNAME      database server host (default: \"local socket\")\n");
-					fprintf(stderr, "  -p, --port=PORT          database server port (default: \"5432\")\n");
-					fprintf(stderr, "  -U, --username=USERNAME  database user name\n");
-					fprintf(stderr, "  -W, --password           force password prompt\n");
-					fprintf(stderr, "\nDebug options:\n");
-					fprintf(stderr, "  --log=FILE               log debug info to file\n");
-					fprintf(stderr, "  --wait=NUM               wait NUM seconds to allow attach from a debugger\n");
-					fprintf(stderr, "\n");
-					fprintf(stderr, "pspg shares lot of key commands with less pager or vi editor.\n");
+					fprintf(stdout, "pspg is a Unix pager designed for table browsing.\n\n");
+					fprintf(stdout, "Usage:\n");
+					fprintf(stdout, "  %s [OPTION] [file]\n", argv[0]);
+					fprintf(stdout, "\nGeneral options:\n");
+					fprintf(stdout, "  --about                  about authors\n");
+					fprintf(stdout, "  --help                   show this help\n");
+					fprintf(stdout, "  -V, --version            show version\n");
+					fprintf(stdout, "  -f, --file=FILE          open file\n");
+					fprintf(stdout, "  -F, --quit-if-one-screen\n");
+					fprintf(stdout, "                           quit if content is one screen\n");
+					fprintf(stdout, "  --hold-stream=NUM        can reopen closed FIFO (0, 1, 2)\n");
+					fprintf(stdout, "  --interactive            force interactive mode\n");
+					fprintf(stdout, "  --ignore_file_suffix     don't try to deduce format from file suffix\n");
+					fprintf(stdout, "  --ni                     not interactive mode (only for csv and query)\n");
+					fprintf(stdout, "  --no-watch-file          don't watch inotify event of file\n");
+					fprintf(stdout, "  --no-mouse               don't use own mouse handling\n");
+					fprintf(stdout, "  --no-sigint-search-reset\n");
+					fprintf(stdout, "                           without reset searching on sigint (CTRL C)\n");
+					fprintf(stdout, "  --no_xterm_mouse_mode    don't use optional xterm mouse mode\n");
+					fprintf(stdout, "  --only-for-tables        use std pager when content is not table\n");
+					fprintf(stdout, "  --no-sigint-exit         without exit on sigint(CTRL C or Escape)\n");
+					fprintf(stdout, "  --pgcli-fix              try to fix some pgcli related issues\n");
+					fprintf(stdout, "  --quit-on-f3             exit on F3 like mc viewers\n");
+					fprintf(stdout, "  --rr=ROWNUM              rows reserved for specific purposes\n");
+					fprintf(stdout, "  --stream                 read input forever\n");
+					fprintf(stdout, "  -X, --reprint-on-exit    preserve content after exit\n");
+					fprintf(stdout, "\nOutput format options:\n");
+					fprintf(stdout, "  -a, --ascii              force ascii\n");
+					fprintf(stdout, "  -b, --blackwhite         black-white style\n");
+					fprintf(stdout, "  -s, --style=N            set color style number (0..%d)\n", MAX_STYLE);
+					fprintf(stdout, "  --bold-labels            row, column labels use bold font\n");
+					fprintf(stdout, "  --bold-cursor            cursor use bold font\n");
+					fprintf(stdout, "  --border                 type of borders (0..2)\n");
+					fprintf(stdout, "  --double-header          header separator uses double lines\n");
+					fprintf(stdout, "  --force-uniborder        replace ascii borders by unicode borders\n");
+					fprintf(stdout, "  --ignore-bad-rows        rows with wrong column numbers are ignored\n");
+					fprintf(stdout, "  --null=STRING            STRING used instead NULL\n");
+					fprintf(stdout, "\nSearching options\n");
+					fprintf(stdout, "  -g --hlite-search, -G --HILITE-SEARCH\n");
+					fprintf(stdout, "                           don't highlight lines for searches\n");
+					fprintf(stdout, "  -i --ignore-case         ignore case in searches that do not contain uppercase\n");
+					fprintf(stdout, "  -I --IGNORE-CASE         ignore case in all searches\n");
+					fprintf(stdout, "\nInterface options:\n");
+					fprintf(stdout, "  -c, --freezecols=N       freeze N columns (0..9)\n");
+					fprintf(stdout, "  --less-status-bar        status bar like less pager\n");
+					fprintf(stdout, "  --line-numbers           show line number column\n");
+					fprintf(stdout, "  --no-bars, --no-commandbar, --no-topbar\n");
+					fprintf(stdout, "                           don't show bottom, top bar or both\n");
+					fprintf(stdout, "  --no-cursor              row cursor will be hidden\n");
+					fprintf(stdout, "  --no-sound               don't use beep when scroll is not possible\n");
+					fprintf(stdout, "  --tabular-cursor         cursor is visible only when data has table format\n");
+					fprintf(stdout, "  --vertical-cursor        show vertical column cursor\n");
+					fprintf(stdout, "\nInput format options:\n");
+					fprintf(stdout, "  --csv                    input stream has csv format\n");
+					fprintf(stdout, "  --csv-separator          char used as field separator\n");
+					fprintf(stdout, "  --csv-header [on/off]    specify header line usage\n");
+					fprintf(stdout, "  --skip-columns-like=\"SPACE SEPARATED STRING LIST\"\n");
+					fprintf(stdout, "                           columns with substr in name are ignored\n");
+					fprintf(stdout, "  --tsv                    input stream has tsv format\n");
+					fprintf(stdout, "\nWatch mode options:\n");
+					fprintf(stdout, "  -q, --query=QUERY        execute query\n");
+					fprintf(stdout, "  -w, --watch time         the query (or read file) is repeated every time (sec)\n");
+					fprintf(stdout, "\nConnection options\n");
+					fprintf(stdout, "  -d, --dbname=DBNAME      database name\n");
+					fprintf(stdout, "  -h, --host=HOSTNAME      database server host (default: \"local socket\")\n");
+					fprintf(stdout, "  -p, --port=PORT          database server port (default: \"5432\")\n");
+					fprintf(stdout, "  -U, --username=USERNAME  database user name\n");
+					fprintf(stdout, "  -W, --password           force password prompt\n");
+					fprintf(stdout, "\nDebug options:\n");
+					fprintf(stdout, "  --log=FILE               log debug info to file\n");
+					fprintf(stdout, "  --wait=NUM               wait NUM seconds to allow attach from a debugger\n");
+					fprintf(stdout, "\n");
+					fprintf(stdout, "pspg shares lot of key commands with less pager or vi editor.\n");
 
 					return false;
 				}
@@ -469,6 +478,13 @@ readargs(char **argv,
 			case 38:
 				opts->csv_skip_columns_like = sstrdup(optarg);
 				break;
+			case 39:
+				opts->pgcli_fix = true;
+				break;
+
+			case 40:
+				opts->xterm_mouse_mode =  false;
+				break;
 
 			case 'V':
 				fprintf(stdout, "pspg-%s\n", PSPG_VERSION);
