@@ -3,7 +3,7 @@
  * commands.c
  *	  a list of commands and translations between keys and commands
  *
- * Portions Copyright (c) 2017-2020 Pavel Stehule
+ * Portions Copyright (c) 2017-2021 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/commands.c
@@ -63,7 +63,6 @@ initialize_special_keycodes()
 
 	CTRL_HOME = get_code("kHOM5", 538);
 	CTRL_END = get_code("kEND5", 533);
-
 }
 
 /*
@@ -164,6 +163,11 @@ cmd_string(int cmd)
 			return "SetTheme_GruvboxLight";
 		case cmd_SetTheme_TaoLight:
 			return "SetTheme_TaoLight";
+		case cmd_SetTheme_Flatwhite:
+			return "SetTheme_Flatwhite";
+		case cmd_SetTheme_RelationalPipes:
+			return "SetThemeRelationalPipes";
+
 		case cmd_SaveSetup:
 			return "SaveSetup";
 
@@ -219,6 +223,8 @@ cmd_string(int cmd)
 			return "ShowLastCol";
 		case cmd_SaveData:
 			return "SaveData";
+		case cmd_SaveAsCSV:
+			return "SaveAsCSV";
 		case cmd_ForwardSearch:
 			return "ForwardSearch";
 		case cmd_BackwardSearch:
@@ -261,6 +267,42 @@ cmd_string(int cmd)
 		case cmd_Refresh:
 			return "Refresh";
 
+		case cmd_SetCopyFile:
+			return "SetCopyFile";
+		case cmd_SetCopyClipboard:
+			return "SetCopyClipboard";
+
+		case cmd_UseClipboard_CSV:
+			return "UseClipboardFormatCSV";
+		case cmd_UseClipboard_TSVC:
+			return "UseClipboardFormatTSVC";
+		case cmd_UseClipboard_text:
+			return "UseClipboardFormattext";
+		case cmd_UseClipboard_INSERT:
+			return "UseClipboardFormatINSERT";
+		case cmd_UseClipboard_INSERT_with_comments:
+			return "UseClipboardFormatINSERTwithcomments";
+		case cmd_TogleEmptyStringIsNULL:
+			return "TogleEmptyStringIsNULL";
+		case cmd_Copy:
+			return "Copy";
+		case cmd_CopyAllLines:
+			return "CopyAllLines";
+		case cmd_CopyTopLines:
+			return "CopyTopLines";
+		case cmd_CopyBottomLines:
+			return "CopyBottomLines";
+		case cmd_CopyLine:
+			return "CopyLine";
+		case cmd_CopyLineExtended:
+			return "CopyLineExtended";
+		case cmd_CopyColumn:
+			return "CopyColumn";
+		case cmd_CopyMarkedLines:
+			return "CopyMarkedLines";
+		case cmd_CopySearchedLines:
+			return "CopySearchedLines";
+
 		default:
 			return "unknown command";
 	}
@@ -321,6 +363,8 @@ translate_event(int c, bool alt, Options *opts)
 				if (opts->quit_on_f3)
 					return cmd_Quit;
 				break;
+			case KEY_IC:
+				return cmd_Copy;
 			case KEY_UP:
 			case 'k':
 				return cmd_CursorUp;
@@ -468,7 +512,10 @@ theme_get_cmd(int theme)
 			return cmd_SetTheme_GruvboxLight;
 		case 20:
 			return cmd_SetTheme_TaoLight;
-
+		case 21:
+			return cmd_SetTheme_Flatwhite;
+		case 22:
+			return cmd_SetTheme_RelationalPipes;
 	};
 
 	return cmd_Invalid;
@@ -525,6 +572,10 @@ cmd_get_theme(int cmd)
 			return 19;
 		case cmd_SetTheme_TaoLight:
 			return 20;
+		case cmd_SetTheme_Flatwhite:
+			return 21;
+		case cmd_SetTheme_RelationalPipes:
+			return 22;
 	};
 
 	return 1;
