@@ -3,7 +3,7 @@
  * command.h
  *	  a list of commands and translations between keys and commands
  *
- * Portions Copyright (c) 2017-2019 Pavel Stehule
+ * Portions Copyright (c) 2017-2021 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/commands.h
@@ -14,6 +14,8 @@
 #ifndef COMMANDS_H
 #define COMMANDS_H
 
+#include "config.h"
+
 /*
  * List of supported commands. Some these commands can be mapped to keys
  */
@@ -65,6 +67,8 @@ typedef enum PspgCommand
 	cmd_SetTheme_SolarLight,
 	cmd_SetTheme_GruvboxLight,
 	cmd_SetTheme_TaoLight,
+	cmd_SetTheme_Flatwhite,
+	cmd_SetTheme_RelationalPipes,
 	cmd_SaveSetup,
 
 	cmd_Escape,
@@ -93,6 +97,7 @@ typedef enum PspgCommand
 	cmd_ShowFirstCol,
 	cmd_ShowLastCol,
 	cmd_SaveData,
+	cmd_SaveAsCSV,
 	cmd_ForwardSearch,
 	cmd_BackwardSearch,
 	cmd_SearchNext,
@@ -109,12 +114,30 @@ typedef enum PspgCommand
 	cmd_SortAsc,
 	cmd_SortDesc,
 	cmd_OriginalSort,
-	cmd_TogglePause
+	cmd_TogglePause,
+	cmd_Refresh,
+	cmd_SetCopyFile,
+	cmd_SetCopyClipboard,
+	cmd_UseClipboard_CSV,
+	cmd_UseClipboard_TSVC,
+	cmd_UseClipboard_text,
+	cmd_UseClipboard_INSERT,
+	cmd_UseClipboard_INSERT_with_comments,
+	cmd_TogleEmptyStringIsNULL,
+	cmd_Copy,
+	cmd_CopyAllLines,
+	cmd_CopyTopLines,
+	cmd_CopyBottomLines,
+	cmd_CopyLine,
+	cmd_CopyLineExtended,
+	cmd_CopyColumn,
+	cmd_CopyMarkedLines,
+	cmd_CopySearchedLines
 } PspgCommand;
 
 extern void initialize_special_keycodes();
 extern const char *cmd_string(int cmd);
-extern int translate_event(int c, bool alt, bool watch_mode);
+extern int translate_event(int c, bool alt, Options *opts);
 
 extern int cmd_get_theme(int cmd);
 extern int theme_get_cmd(int theme);
