@@ -3,7 +3,7 @@
  * config.c
  *	  a routines for loading, saving configuration
  *
- * Portions Copyright (c) 2017-2020 Pavel Stehule
+ * Portions Copyright (c) 2017-2021 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/config.c
@@ -110,6 +110,8 @@ save_config(char *path, Options *opts)
 	SAFE_SAVE_BOOL_OPTION("no_sigint_search_reset", opts->no_sigint_search_reset);
 	SAFE_SAVE_BOOL_OPTION("double_header", opts->double_header);
 	SAFE_SAVE_BOOL_OPTION("quit_on_f3", opts->quit_on_f3);
+	SAFE_SAVE_BOOL_OPTION("pgcli_fix", opts->pgcli_fix);
+	SAFE_SAVE_BOOL_OPTION("xterm_mouse_mode", opts->xterm_mouse_mode);
 
 	result = fprintf(f, "theme = %d\n", opts->theme);
 	if (result < 0)
@@ -119,6 +121,10 @@ save_config(char *path, Options *opts)
 	if (result < 0)
 		return false;
 
+	result = fprintf(f, "default_clipboard_format = %d\n", opts->clipboard_format);
+	if (result < 0)
+		return false;
+
 	result = fclose(f);
 	if (result != 0)
 		return false;
@@ -178,7 +184,10 @@ load_config(char *path, Options *opts)
 			else if (strcmp(key, "show_rownum") == 0)
 				opts->show_rownum = bool_val;
 			else if (strcmp(key, "theme") == 0)
-				opts->theme = int_val;
+			{
+				if (int_val >= 0 && int_val <= MAX_STYLE)
+					opts->theme = int_val;
+			}
 			else if (strcmp(key, "without_commandbar") == 0)
 				opts->no_commandbar = bool_val;
 			else if (strcmp(key, "without_topbar") == 0)
@@ -195,6 +204,15 @@ load_config(char *path, Options *opts)
 				opts->no_sigint_search_reset = bool_val;
 			else if (strcmp(key, "quit_on_f3") == 0)
 				opts->quit_on_f3 = bool_val;
+			else if (strcmp(key, "pgcli_fix") == 0)
+				opts->pgcli_fix = bool_val;
+			else if (strcmp(key, "default_clipboard_format") == 0)
+			{
+				if (int_val > 0 && int_val <= CLIPBOARD_FORMAT_INSERT_WITH_COMMENTS)
+					opts->clipboard_format = int_val;
+			}
+			else if (strcmp(key, "xterm_mouse_mode") == 0)
+				opts->xterm_mouse_mode = bool_val;
 
 			free(line);
 			line = NULL;
