@@ -3,7 +3,7 @@
  * config.h
  *	  load/save configuration
  *
- * Portions Copyright (c) 2017-2020 Pavel Stehule
+ * Portions Copyright (c) 2017-2021 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/config.h
@@ -16,6 +16,26 @@
 
 #include <stdbool.h>
 
+#define MAX_STYLE					22
+
+typedef enum
+{
+	CLIPBOARD_FORMAT_CSV,
+	CLIPBOARD_FORMAT_TSVC,
+	CLIPBOARD_FORMAT_TEXT,
+	CLIPBOARD_FORMAT_INSERT,
+	CLIPBOARD_FORMAT_INSERT_WITH_COMMENTS
+} ClipboardFormat;
+
+#define DSV_FORMAT_TYPE(f)		(f == CLIPBOARD_FORMAT_CSV || f == CLIPBOARD_FORMAT_TSVC)
+#define INSERT_FORMAT_TYPE(f)	(f == CLIPBOARD_FORMAT_INSERT || f == CLIPBOARD_FORMAT_INSERT_WITH_COMMENTS)
+
+typedef enum
+{
+	COPY_TARGET_FILE,
+	COPY_TARGET_CLIPBOARD
+} CopyTarget;
+
 typedef struct
 {
 	char   *pathname;
@@ -47,6 +67,7 @@ typedef struct
 	char   *nullstr;
 	char   *csv_skip_columns_like;
 	bool	ignore_short_rows;
+	bool	pgcli_fix;			/* hints for using from pgcli */
 	char	double_header;
 	int		border_type;
 	bool	on_sigint_exit;
@@ -61,6 +82,10 @@ typedef struct
 	char   *dbname;
 	bool	watch_file;
 	bool	quit_on_f3;
+	ClipboardFormat clipboard_format;
+	CopyTarget copy_target;
+	bool	empty_string_is_null;
+	bool	xterm_mouse_mode;
 } Options;
 
 extern bool save_config(char *path, Options *opts);
