@@ -3,7 +3,7 @@
  * infra.c
  *	  a routines for build a infrastructure
  *
- * Portions Copyright (c) 2017-2020 Pavel Stehule
+ * Portions Copyright (c) 2017-2021 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/infra.c
@@ -73,11 +73,14 @@ leave(const char *fmt, ...)
 {
 	va_list		args;
 
-	if (active_ncurses)
-		endwin();
+	exit_ncurses();
 
 	if (!fmt)
+	{
+		if (current_state && current_state->logfile)
+			fclose(current_state->logfile);
 		exit(EXIT_FAILURE);
+	}
 
 	va_start(args, fmt);
 	vfprintf(stderr, fmt, args);
@@ -94,6 +97,7 @@ leave(const char *fmt, ...)
 		va_end(args);
 
 		fputc('\n', current_state->logfile);
+		fclose(current_state->logfile);
 	}
 
 #ifdef DEBUG_PIPE
@@ -177,3 +181,18 @@ sstrdup2(char *str, char *debugstr)
 
 	return result;
 }
+
+char *
+sstrndup(char *str, int bytes)
+{
+	char   *result, *ptr;
+
+	result = ptr = smalloc(bytes + 1);
+
+	while (*str && bytes-- > 0)
+		*ptr++ = *str++;
+
+	*ptr = '\0';
+
+	return result;
+}
\ No newline at end of file
