@@ -3,7 +3,7 @@
  * menu.c
  *	 holds menu related code
  *
- * Portions Copyright (c) 2017-2020 Pavel Stehule
+ * Portions Copyright (c) 2017-2021 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/menu.c
@@ -17,8 +17,9 @@
 #include "st_menu.h"
 #include "commands.h"
 
-#define MENU_ITEM_THEME				1
-#define MENU_ITEM_OPTIONS			2
+#define MENU_ITEM_THEME				10
+#define MENU_ITEM_OPTIONS			11
+#define MENU_ITEM_COPY				12
 
 ST_CMDBAR_ITEM _bottombar[] = {
 	{"Save", false, 2, cmd_SaveData, 0},
@@ -31,14 +32,44 @@ ST_CMDBAR_ITEM _bottombar[] = {
 ST_CMDBAR_ITEM _bottombar_alt1[] = {
 	{"Save", false, 2, cmd_SaveData, 0},
 	{"Quit", false, 3, cmd_Quit, 0},
+	{"Copy", false, 5, 0, 0},
 	{"Search", false, 7, cmd_ForwardSearch, 0},
 	{"Menu", false, 9, cmd_ShowMenu, 0},
 	{"Quit", false, 10, cmd_Quit, 0},
 	{NULL, false, 0, 0, 0}
 };
 
+ST_MENU_ITEM _copy[] = {
+	{"~C~opy", cmd_Copy, "Ins", 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Copy ~l~ine", cmd_CopyLine, NULL, 0, 0, 0, NULL},
+	{"Copy line e~x~tended", cmd_CopyLineExtended, NULL, 0, 0, 0, NULL},
+	{"Copy col~u~mn", cmd_CopyColumn, NULL, 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Copy ~a~ll", cmd_CopyAllLines, NULL, 0, 0, 0, NULL},
+	{"Copy ~t~op lines", cmd_CopyTopLines, NULL, 0, 0, 0, NULL},
+	{"Copy ~b~ottom lines", cmd_CopyBottomLines, NULL, 0, 0, 0, NULL},
+	{"Copy ~m~arked lines", cmd_CopyMarkedLines, NULL, 0, 0, 0, NULL},
+	{"Copy ~s~earched lines", cmd_CopySearchedLines, NULL, 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Copy to ~f~ile", cmd_SetCopyFile, NULL, 0, 0, 0, NULL},
+	{"Copy to cli~p~board", cmd_SetCopyClipboard, NULL, 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Empty string is NULL", cmd_TogleEmptyStringIsNULL, NULL, 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"_0_Use CSV format", cmd_UseClipboard_CSV, NULL, 0, 0, 0, NULL},
+	{"_1_Use LibreOffice TSVC format", cmd_UseClipboard_TSVC, NULL, 0, 0, 0, NULL},
+	{"_2_Use formatted text", cmd_UseClipboard_text, NULL, 0, 0, 0, NULL},
+	{"_3_Use INSERT format", cmd_UseClipboard_INSERT, NULL, 0, 0, 0, NULL},
+	{"_4_Use commented INSERT format", cmd_UseClipboard_INSERT_with_comments, NULL, 0, 0, 0, NULL},
+	{NULL, 0, NULL, 0, 0, 0, NULL}
+};
+
 ST_MENU_ITEM _file[] = {
+	{"~C~opy to", 0, NULL, 0, 0, 0, _copy},
+	{"--", 0, NULL, 0, 0, 0, NULL},
 	{"~S~ave", cmd_SaveData, "s", 0, 0, 0, NULL},
+	{"Sa~v~e as CSV", cmd_SaveAsCSV, NULL, 0, 0, 0, NULL},
 	{"--", 0, NULL, 0, 0, 0, NULL},
 	{"~R~aw output quit", cmd_RawOutputQuit, "M-q", 0, 0, 0, NULL},
 	{"E~x~it", cmd_Quit, "q, F10", 0, 0, 0, NULL},
@@ -116,6 +147,8 @@ ST_MENU_ITEM _theme[] = {
 	{"_g_Solar Light theme", cmd_SetTheme_SolarLight, NULL, 0, 0, 0, NULL},
 	{"_u_Gruvbox Light theme", cmd_SetTheme_GruvboxLight, NULL, 0, 0, 0, NULL},
 	{"_t_Tao Light theme", cmd_SetTheme_TaoLight, NULL, 0, 0, 0, NULL},
+	{"_f_Flatwhite theme", cmd_SetTheme_Flatwhite, NULL, 0, 0, 0, NULL},
+	{"_a_Relational Pipes theme", cmd_SetTheme_RelationalPipes, NULL, 0, 0, 0, NULL},
 	{NULL},
 };
 
@@ -155,12 +188,10 @@ ST_MENU_ITEM menubar[] = {
   {NULL}
 };
 
-
 ST_MENU_CONFIG		menu_config;
 ST_MENU_CONFIG		menu_config2;
 int menu_theme = -1;
 
-
 /*
  * Returns menu style joined to main theme
  */
@@ -205,6 +236,10 @@ get_menu_style(int main_theme)
 			return ST_MENU_STYLE_ONECOLOR;
 		case 20:
 			return ST_MENU_STYLE_TAO;
+		case 21:
+			return ST_MENU_STYLE_FLATWHITE;
+		case 22:
+			return ST_MENU_STYLE_DBASE;
 		default:
 			return ST_MENU_STYLE_VISION;
 	}
@@ -218,37 +253,39 @@ init_menu_config(Options *opts)
 {
 	int start_from_rgb = 220;
 
-	menu_config.force8bit = opts->force8bit;
-	menu_config.force_ascii_art = opts->force_ascii_art;
-	menu_config.language = NULL;
-	menu_config.encoding = NULL;
-
-	menu_config2.force8bit = opts->force8bit;
-	menu_config2.language = NULL;
-	menu_config2.encoding = NULL;
-
 	menu_theme = get_menu_style(opts->theme);
 
 	if (menu_theme == ST_MENU_STYLE_FREE_DOS)
 	{
 		int		fcp;
 
-		fcp = st_menu_load_style(&menu_config, menu_theme, 30);
-		st_menu_load_style(&menu_config2, ST_MENU_STYLE_FREE_DOS_P, fcp);
-	}
-		st_menu_load_style_rgb(&menu_config, menu_theme,
-								menu_theme == ST_MENU_STYLE_ONECOLOR ? 3 : 30, &start_from_rgb);
+		fcp = st_menu_load_style(&menu_config,
+								 menu_theme,
+								 30,
+								 opts->force8bit,
+								 opts->force_ascii_art);
 
+		st_menu_load_style(&menu_config2,
+						   ST_MENU_STYLE_FREE_DOS_P,
+						   fcp,
+						   opts->force8bit,
+						   opts->force_ascii_art);
+	}
+	else
+		st_menu_load_style_rgb(&menu_config,
+							   menu_theme,
+							   menu_theme == ST_MENU_STYLE_ONECOLOR ? 3 : 30,
+							   &start_from_rgb,
+							   opts->force8bit,
+							   opts->force_ascii_art);
+
+	/* extra pspg menu theme customization */
 	if (opts->theme == 1)
 		menu_config.shadow_width = 2;
 	else if (opts->theme == 4)
 		menu_config.text_space = 4;
-
-	menu_config.force_ascii_art = opts->force_ascii_art;
-	menu_config2.force_ascii_art = opts->force_ascii_art;
 }
 
-
 /*
  * Prepare configuration and initialize menu
  */
@@ -331,4 +368,27 @@ post_menu(Options *opts, struct ST_MENU *menu)
 
 	st_menu_reset_all_submenu_options(menu, MENU_ITEM_THEME, ST_MENU_OPTION_MARKED);
 	st_menu_enable_option(menu, theme_get_cmd(opts->theme), ST_MENU_OPTION_MARKED);
+
+	refresh_copy_target_options(opts, menu);
+	refresh_clipboard_options(opts, menu);
+
+	st_menu_set_option(menu, cmd_TogleEmptyStringIsNULL, ST_MENU_OPTION_MARKED, opts->empty_string_is_null);
+}
+
+void
+refresh_clipboard_options(Options *opts, struct ST_MENU *menu)
+{
+	st_menu_set_option(menu, cmd_UseClipboard_CSV, ST_MENU_OPTION_MARKED, opts->clipboard_format == CLIPBOARD_FORMAT_CSV);
+	st_menu_set_option(menu, cmd_UseClipboard_TSVC, ST_MENU_OPTION_MARKED, opts->clipboard_format == CLIPBOARD_FORMAT_TSVC);
+	st_menu_set_option(menu, cmd_UseClipboard_text, ST_MENU_OPTION_MARKED, opts->clipboard_format == CLIPBOARD_FORMAT_TEXT);
+	st_menu_set_option(menu, cmd_UseClipboard_INSERT, ST_MENU_OPTION_MARKED, opts->clipboard_format == CLIPBOARD_FORMAT_INSERT);
+	st_menu_set_option(menu, cmd_UseClipboard_INSERT_with_comments, ST_MENU_OPTION_MARKED,
+					   opts->clipboard_format == CLIPBOARD_FORMAT_INSERT_WITH_COMMENTS);
+}
+
+void
+refresh_copy_target_options(Options *opts, struct ST_MENU *menu)
+{
+	st_menu_set_option(menu, cmd_SetCopyFile, ST_MENU_OPTION_MARKED, opts->copy_target == COPY_TARGET_FILE);
+	st_menu_set_option(menu, cmd_SetCopyClipboard, ST_MENU_OPTION_MARKED, opts->copy_target == COPY_TARGET_CLIPBOARD);
 }
