@@ -3,7 +3,7 @@
  * pgcliend.c
  *	  execute query and format result
  *
- * Portions Copyright (c) 2017-2020 Pavel Stehule
+ * Portions Copyright (c) 2017-2021 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/pgclient.c
@@ -397,6 +397,8 @@ pg_exec_query(Options *opts, RowBucketType *rb, PrintDataDesc *pdesc, const char
 			EXIT_OUT_OF_MEMORY();
 	}
 
+	free(hidden);
+
 	PQclear(result);
 	PQfinish(conn);
 
@@ -406,6 +408,9 @@ pg_exec_query(Options *opts, RowBucketType *rb, PrintDataDesc *pdesc, const char
 
 #else
 
+	(void) rb;
+	(void) pdesc;
+
 	*err = "Query cannot be executed. The Postgres library was not available at compile time.";
 
 	return false;
