@@ -3,7 +3,7 @@
  * pretty-csv.c
  *	  import and formatting csv and tsv documents
  *
- * Portions Copyright (c) 2017-2019 Pavel Stehule
+ * Portions Copyright (c) 2017-2021 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/pretty-csv.c
@@ -38,8 +38,9 @@ typedef struct
 	long int	digits[1024];		/* number of digits, used for format detection */
 	long int	tsizes[1024];		/* size of column in bytes, used for format detection */
 	int			firstdigit[1024];	/* rows where first char is digit */
-	int			widths[1024];			/* column's display width */
+	size_t		widths[1024];			/* column's display width */
 	bool		multilines[1024];		/* true if column has multiline row */
+	bool		hidden[1024];
 } LinebufType;
 
 typedef struct
@@ -64,24 +65,6 @@ typedef struct
 	bool		ignore_short_rows;
 } PrintConfigType;
 
-static void *
-smalloc(int size, char *debugstr)
-{
-	char *result;
-
-	result = malloc(size);
-	if (!result)
-	{
-		if (debugstr)
-			fprintf(stderr, "out of memory while %s\n", debugstr);
-		else
-			fprintf(stderr, "out of memory\n");
-		exit(EXIT_FAILURE);
-	}
-
-	return result;
-}
-
 /*
  * Add new row to LineBuffer
  */
@@ -92,7 +75,7 @@ pb_flush_line(PrintbufType *printbuf)
 
 	if (printbuf->linebuf->nrows == 1000)
 	{
-		LineBuffer *nb = smalloc(sizeof(LineBuffer), "serialize csv output");
+		LineBuffer *nb = smalloc2(sizeof(LineBuffer), "serialize csv output");
 
 		memset(nb, 0, sizeof(LineBuffer));
 
@@ -101,7 +84,7 @@ pb_flush_line(PrintbufType *printbuf)
 		printbuf->linebuf = nb;
 	}
 
-	line = smalloc(printbuf->used + 1, "serialize csv output");
+	line = smalloc2(printbuf->used + 1, "serialize csv output");
 	memcpy(line, printbuf->buffer, printbuf->used);
 	line[printbuf->used] = '\0';
 
@@ -126,7 +109,7 @@ pb_write(PrintbufType *printbuf, const char *str, int size)
 
 		printbuf->buffer = realloc(printbuf->buffer, printbuf->size);
 		if (!printbuf->buffer)
-			leave_ncurses("out of memory while serialize csv output\n");
+			leave("out of memory while serialize csv output");
 	}
 
 	memcpy(printbuf->buffer + printbuf->used, str, size);
@@ -158,7 +141,7 @@ pb_write_repeat(PrintbufType *printbuf, int n, const char *str, int size)
 	{
 		printbuf->buffer = realloc(printbuf->buffer, printbuf->size);
 		if (!printbuf->buffer)
-			leave_ncurses("out of memory while serialize csv output\n");
+			leave("out of memory while serialize csv output");
 	}
 
 	while (n--)
@@ -186,7 +169,7 @@ pb_putc(PrintbufType *printbuf, char c)
 
 		printbuf->buffer = realloc(printbuf->buffer, printbuf->size);
 		if (!printbuf->buffer)
-			leave_ncurses("out of memory while serialize csv output\n");
+			leave("out of memory while serialize csv output");
 	}
 
 	printbuf->free -= 1;
@@ -216,7 +199,7 @@ pb_putc_repeat(PrintbufType *printbuf, int n, int c)
 	{
 		printbuf->buffer = realloc(printbuf->buffer, printbuf->size);
 		if (!printbuf->buffer)
-			leave_ncurses("out of memory while serialize csv output\n");
+			leave("out of memory while serialize csv output");
 	}
 
 	memset(printbuf->buffer + printbuf->used, c, n);
@@ -450,34 +433,17 @@ pb_print_rowbuckets(PrintbufType *printbuf,
 			int		j;
 			bool	isheader = false;
 			RowType	   *row;
-			bool	free_row;
 			bool	more_lines = true;
 			bool	multiline = rb->multilines[i];
+			char	   *fields[1024];
+			int			multiline_lineno;
 
 			/* skip broken rows */
-			if (pconfig->ignore_short_rows && rb->rows[i]->nfields != pdesc->nfields)
+			if (pconfig->ignore_short_rows && rb->rows[i]->nfields != pdesc->nfields_all)
 				continue;
 
-			/*
-			 * For multilines we can modify pointers so do copy now
-			 */
-			if (multiline)
-			{
-				RowType	   *source = rb->rows[i];
-				int			size;
-
-				size = offsetof(RowType, fields) + (source->nfields * sizeof(char*));
-
-				row = smalloc(size, "RowType");
-				memcpy(row, source, size);
-
-				free_row = true;
-			}
-			else
-			{
-				row = rb->rows[i];
-				free_row = false;
-			}
+			multiline_lineno = 0;
+			row = rb->rows[i];
 
 			while (more_lines)
 			{
@@ -495,7 +461,7 @@ pb_print_rowbuckets(PrintbufType *printbuf,
 
 				isheader = printed_rows == 0 ? pdesc->has_header : false;
 
-				for (j = 0; j < row->nfields; j++)
+				for (j = 0; j < pdesc->nfields; j++)
 				{
 					int		width;
 					int		spaces;
@@ -513,7 +479,18 @@ pb_print_rowbuckets(PrintbufType *printbuf,
 						}
 					}
 
-					field = row->fields[j];
+					if (pdesc->columns_map[j] < row->nfields)
+					{
+						if (multiline_lineno == 0)
+						{
+							field = row->fields[pdesc->columns_map[j]];
+							fields[j] = NULL;
+						}
+						else
+							field = fields[j];
+					}
+					else
+						field = NULL;
 
 					if (field && *field != '\0')
 					{
@@ -567,9 +544,9 @@ pb_print_rowbuckets(PrintbufType *printbuf,
 							pb_putc_repeat(printbuf, spaces, ' ');
 
 						if (multiline)
-							row->fields[j] = pb_put_line(row->fields[j], multiline, printbuf);
+							fields[j] = pb_put_line(field, multiline, printbuf);
 						else
-							(void) pb_put_line(row->fields[j], multiline, printbuf);
+							(void) pb_put_line(field, multiline, printbuf);
 
 						/* right spaces */
 						if (isheader)
@@ -594,26 +571,6 @@ pb_print_rowbuckets(PrintbufType *printbuf,
 					}
 				}
 
-				for (j = row->nfields; j < pdesc->nfields; j++)
-				{
-					bool	addspace;
-
-					if (j > 0)
-					{
-						if (border != 0)
-						{
-							if (linestyle == 'a')
-								pb_write(printbuf, "| ", 2);
-							else
-								pb_write(printbuf, "\342\224\202 ", 4);
-						}
-					}
-
-					addspace = border != 0 || j < last_column_num || is_last_column_multiline;
-
-					pb_putc_repeat(printbuf, pdesc->widths[j] + (addspace ? 1 : 0), ' ');
-				}
-
 				if (border == 2)
 				{
 					if (linestyle == 'a')
@@ -631,10 +588,8 @@ pb_print_rowbuckets(PrintbufType *printbuf,
 				}
 
 				printed_rows += 1;
+				multiline_lineno += 1;
 			}
-
-			if (free_row)
-				free(row);
 		}
 
 		rb = rb->next_bucket;
@@ -655,13 +610,18 @@ prepare_pdesc(RowBucketType *rb, LinebufType *linebuf, PrintDataDesc *pdesc, Pri
 {
 	int				i;
 
-	/* copy data from linebuf */
-	pdesc->nfields = linebuf->maxfields;
+	pdesc->nfields_all = linebuf->maxfields;
+	pdesc->nfields = 0;
 
-	for (i = 0; i < pdesc->nfields; i++)
+	/* copy data from linebuf */
+	for (i = 0; i < linebuf->maxfields; i++)
 	{
-		pdesc->widths[i] = linebuf->widths[i];
-		pdesc->multilines[i] = linebuf->multilines[i];
+		if (!linebuf->hidden[i])
+		{
+			pdesc->widths[pdesc->nfields] = linebuf->widths[i];
+			pdesc->multilines[pdesc->nfields] = linebuf->multilines[i];
+			pdesc->columns_map[pdesc->nfields++] = i;
+		}
 	}
 
 	if (pconfig->header_mode == 'a')
@@ -672,11 +632,11 @@ prepare_pdesc(RowBucketType *rb, LinebufType *linebuf, PrintDataDesc *pdesc, Pri
 	/* try to detect types from numbers of digits */
 	for (i = 0; i < pdesc->nfields; i++)
 	{
-		if ((linebuf->tsizes[i] == 0 && linebuf->digits[i] > 0) ||
-			(linebuf->firstdigit[i] > 0 && linebuf->processed - 1 == 1))
+		if ((linebuf->tsizes[pdesc->columns_map[i]] == 0 && linebuf->digits[pdesc->columns_map[i]] > 0) ||
+			(linebuf->firstdigit[pdesc->columns_map[i]] > 0 && linebuf->processed - 1 == 1))
 			pdesc->types[i] = 'd';
-		else if ((((double) linebuf->firstdigit[i] / (double) (linebuf->processed - 1)) > 0.8)
-			&& (((double) linebuf->digits[i] / (double) linebuf->tsizes[i]) > 0.5))
+		else if ((((double) linebuf->firstdigit[pdesc->columns_map[i]] / (double) (linebuf->processed - 1)) > 0.8)
+			&& (((double) linebuf->digits[pdesc->columns_map[i]] / (double) linebuf->tsizes[pdesc->columns_map[i]]) > 0.5))
 			pdesc->types[i] = 'd';
 		else
 			pdesc->types[i] = 'a';
@@ -695,7 +655,7 @@ append_char(LinebufType *linebuf, char c)
 		linebuf->buffer = realloc(linebuf->buffer, linebuf->size);
 
 		if (!linebuf->buffer)
-			leave_ncurses("out of memory while read csv or tsv data\n");
+			leave("out of memory while read csv or tsv data");
 	}
 
 	linebuf->buffer[linebuf->used++] = c;
@@ -715,7 +675,7 @@ append_str(LinebufType *linebuf, char *str)
 		linebuf->buffer = realloc(linebuf->buffer, linebuf->size);
 
 		if (!linebuf->buffer)
-			leave_ncurses("out of memory while read csv or tsv data\n");
+			leave("out of memory while read csv or tsv data");
 	}
 
 	while (*str)
@@ -732,7 +692,7 @@ prepare_RowBucket(RowBucketType *rb)
 	/* move row from linebuf to rowbucket */
 	if (rb->nrows >= 1000)
 	{
-		RowBucketType *new = smalloc(sizeof(RowBucketType), "import csv data");
+		RowBucketType *new = smalloc2(sizeof(RowBucketType), "import csv data");
 
 		new->nrows = 0;
 		new->allocated = true;
@@ -758,7 +718,7 @@ postprocess_fields(int nfields,
 				   bool *is_multiline_row)
 {
 	bool		malformed;
-	int			width;
+	size_t		width;
 	int		i;
 
 	if (ignore_short_rows)
@@ -774,10 +734,14 @@ postprocess_fields(int nfields,
 		long int	total = 0;
 		bool		multiline;
 
+		/* don't calculate width for hidden columns */
+		if (linebuf->hidden[i])
+			continue;
+
 		if (force8bit)
 		{
-			int		cw = 0;
-			char   *ptr = row->fields[i];
+			size_t		cw = 0;
+			char	    *ptr = row->fields[i];
 
 			width = 0;
 
@@ -845,8 +809,8 @@ postprocess_rows(RowBucketType *rb,
 				 bool force8bit,
 				 char *nullstr)
 {
-	int		nullstr_size = strlen(nullstr);
-	int		nullstr_width = force8bit ? strlen(nullstr) : utf_string_dsplen(nullstr, strlen(nullstr));
+	size_t		nullstr_size = strlen(nullstr);
+	size_t		nullstr_width = force8bit ? strlen(nullstr) : (size_t) utf_string_dsplen(nullstr, strlen(nullstr));
 
 	while (rb)
 	{
@@ -880,8 +844,8 @@ postprocess_rows(RowBucketType *rb,
 					}
 				}
 
-				locbuf = smalloc(newsize, "postprocess csv or tsv data");
-				newrow = smalloc(offsetof(RowType, fields) + (linebuf->maxfields * sizeof(char*)),
+				locbuf = smalloc2(newsize, "postprocess csv or tsv data");
+				newrow = smalloc2(offsetof(RowType, fields) + (linebuf->maxfields * sizeof(char*)),
 								 "postprocess csv or tsv data");
 				newrow->nfields = linebuf->maxfields;
 
@@ -919,6 +883,75 @@ postprocess_rows(RowBucketType *rb,
 	}
 }
 
+static bool
+mark_hidden_columns(LinebufType *linebuf,
+					RowType *row,
+					int nfields,
+					Options *opts)
+{
+	char	   *names;
+	char	   *endnames;
+	char	   *ptr;
+	int		i;
+	bool	result = false;
+
+	/* prepare list of hidden columns */
+	names = sstrdup(opts->csv_skip_columns_like);
+
+	endnames = names + strlen(names);
+	ptr = names;
+	while (*ptr)
+	{
+		/* space is separator between words */
+		if (*ptr == ' ')
+			*ptr = '\0';
+		ptr += 1;
+	}
+
+	for (i = 0; i < nfields; i++)
+	{
+		ptr = names;
+		while (ptr < endnames)
+		{
+			if (*ptr)
+			{
+				size_t		len = strlen(ptr);
+
+				if (*ptr == '^')
+				{
+					if (strncmp(row->fields[i], ptr + 1, len - 1) == 0)
+					{
+						linebuf->hidden[i] = true;
+						result = true;
+					}
+				}
+				else if (ptr[len - 1] == '$')
+				{
+					size_t		len2 = strlen(row->fields[i]);
+
+					if (len2 > (len - 1) &&
+						strncmp(row->fields[i] + len2 - len + 1, ptr, len - 1) == 0)
+					{
+						linebuf->hidden[i] = true;
+						result = true;
+					}
+				}
+				else if (strstr(row->fields[i], ptr))
+				{
+					linebuf->hidden[i] = true;
+					result = true;
+				}
+			}
+
+			ptr += strlen(ptr) + 1;
+		}
+	}
+
+	free(names);
+
+	return result;
+}
+
 /*
  * Read tsv format from ifile
  */
@@ -928,13 +961,13 @@ read_tsv(RowBucketType *rb,
 		 bool force8bit,
 		 FILE *ifile,
 		 bool ignore_short_rows,
-		 char *nullstr)
+		 Options *opts)
 {
 	bool	closed = false;
 	int		size = 0;
 	int		nfields = 0;
 	int		c;
-	int		nullstr_size = strlen(nullstr);
+	int		nullstr_size = strlen(opts->nullstr);
 
 	c = fgetc(ifile);
 	do
@@ -957,7 +990,7 @@ read_tsv(RowBucketType *rb,
 					/* NULL */
 					if (c == 'N')
 					{
-						append_str(linebuf, nullstr);
+						append_str(linebuf, opts->nullstr);
 						size += nullstr_size;
 						goto next_char;
 					}
@@ -1014,10 +1047,10 @@ read_tsv(RowBucketType *rb,
 
 				rb = prepare_RowBucket(rb);
 
-				locbuf = smalloc(linebuf->used, "import csv data");
+				locbuf = smalloc2(linebuf->used, "import tsv data");
 				memcpy(locbuf, linebuf->buffer, linebuf->used);
 
-				row = smalloc(offsetof(RowType, fields) + (nfields * sizeof(char*)), "import csv data");
+				row = smalloc2(offsetof(RowType, fields) + (nfields * sizeof(char*)), "import csv data");
 				row->nfields = nfields;
 
 				for (i = 0; i < nfields; i++)
@@ -1026,6 +1059,9 @@ read_tsv(RowBucketType *rb,
 					locbuf += linebuf->sizes[i];
 				}
 
+				if (linebuf->processed == 0 && opts->csv_skip_columns_like)
+					mark_hidden_columns(linebuf, row, nfields, opts);
+
 				postprocess_fields(nfields,
 								   row,
 								   linebuf,
@@ -1036,6 +1072,8 @@ read_tsv(RowBucketType *rb,
 
 				rb->multilines[rb->nrows] = multiline;
 				rb->rows[rb->nrows++] = row;
+
+				linebuf->processed += 1;
 			}
 
 			nfields = 0;
@@ -1052,7 +1090,7 @@ next_char:
 
 	/* append nullstr to missing columns */
 	if (nullstr_size > 0 && !ignore_short_rows)
-		postprocess_rows(rb, linebuf, force8bit, nullstr);
+		postprocess_rows(rb, linebuf, force8bit, opts->nullstr);
 }
 
 static void
@@ -1062,7 +1100,7 @@ read_csv(RowBucketType *rb,
 		 bool force8bit,
 		 FILE *ifile,
 		 bool ignore_short_rows,
-		 char *nullstr)
+		 Options *opts)
 {
 	bool	skip_initial = true;
 	bool	closed = false;
@@ -1073,9 +1111,21 @@ read_csv(RowBucketType *rb,
 	int		nfields = 0;
 	int		instr = false;			/* true when csv string is processed */
 	int		c;
-	int		nullstr_size = strlen(nullstr);
+	int		nullstr_size = strlen(opts->nullstr);
 
 	c = fgetc(ifile);
+
+	if (opts->pgcli_fix && c == '>')
+	{
+		while (c != '\n' && c != EOF)
+		{
+			fputc(c, stdout);
+			c = fgetc(ifile);
+		}
+
+		fputc('\n', stdout);
+	}
+
 	do
 	{
 		/* ignore ^M */
@@ -1143,16 +1193,10 @@ read_csv(RowBucketType *rb,
 			if (sep != -1 && c == sep && !instr)
 			{
 				if (nfields >= 1024)
-				{
-					fprintf(stderr, "too much columns");
-					exit(EXIT_FAILURE);
-				}
+					leave("too much columns");
 
 				if (skip_initial)
-				{
-					fprintf(stderr, "internal error - unexpected value of variable: \"skip_initial\"\n");
-					exit(EXIT_FAILURE);
-				}
+					leave("internal error - unexpected value of variable: \"skip_initial\"");
 
 				if (last_nw - first_nw > 0 || found_string || nullstr_size == 0)
 				{
@@ -1165,7 +1209,7 @@ read_csv(RowBucketType *rb,
 					linebuf->sizes[nfields] = nullstr_size;
 					linebuf->starts[nfields++] = pos;
 
-					append_str(linebuf, nullstr);
+					append_str(linebuf, opts->nullstr);
 					pos += nullstr_size;
 				}
 
@@ -1189,7 +1233,7 @@ read_csv(RowBucketType *rb,
 					c = fgetc(ifile);
 					if (c == EOF)
 					{
-						log_writeln("unexpected quit, broken unicode char\n");
+						log_row("unexpected quit, broken unicode char");
 						break;
 					}
 
@@ -1228,7 +1272,7 @@ read_csv(RowBucketType *rb,
 				linebuf->sizes[nfields] = nullstr_size;
 				linebuf->starts[nfields++] = pos;
 
-				append_str(linebuf, nullstr);
+				append_str(linebuf, opts->nullstr);
 				pos += nullstr_size;
 			}
 			else
@@ -1244,27 +1288,36 @@ read_csv(RowBucketType *rb,
 
 			data_size = 0;
 			for (i = 0; i < nfields; i++)
-				data_size += linebuf->sizes[i] + 1;
+				if (!linebuf->hidden[i])
+					data_size += linebuf->sizes[i] + 1;
 
-			locbuf = smalloc(data_size, "import csv data");
+			locbuf = smalloc2(data_size, "import csv data");
 			memset(locbuf, 0, data_size);
 
-			row = smalloc(offsetof(RowType, fields) + (nfields * sizeof(char*)), "import csv data");
+			row = smalloc2(offsetof(RowType, fields) + (nfields * sizeof(char*)), "import csv data");
 			row->nfields = nfields;
 
 			multiline = false;
 
 			for (i = 0; i < nfields; i++)
 			{
-				row->fields[i] = locbuf;
+				if (!linebuf->hidden[i])
+				{
+					row->fields[i] = locbuf;
 
-				if (linebuf->sizes[i] > 0)
-					memcpy(locbuf, linebuf->buffer + linebuf->starts[i], linebuf->sizes[i]);
+					if (linebuf->sizes[i] > 0)
+						memcpy(locbuf, linebuf->buffer + linebuf->starts[i], linebuf->sizes[i]);
 
-				locbuf[linebuf->sizes[i]] = '\0';
-				locbuf += linebuf->sizes[i] + 1;
+					locbuf[linebuf->sizes[i]] = '\0';
+					locbuf += linebuf->sizes[i] + 1;
+				}
+				else
+					row->fields[i] = NULL;
 			}
 
+			if (linebuf->processed == 0 && opts->csv_skip_columns_like)
+				mark_hidden_columns(linebuf, row, nfields, opts);
+
 			postprocess_fields(nfields,
 							   row,
 							   linebuf,
@@ -1301,7 +1354,7 @@ next_char:
 
 	/* append nullstr to missing columns */
 	if (nullstr_size > 0 && !ignore_short_rows)
-		postprocess_rows(rb, linebuf, force8bit, nullstr);
+		postprocess_rows(rb, linebuf, force8bit, opts->nullstr);
 }
 
 /*
@@ -1309,7 +1362,7 @@ next_char:
  *
  */
 bool
-read_and_format(FILE *fp, Options *opts, DataDesc *desc, const char **err)
+read_and_format(Options *opts, DataDesc *desc, StateData *state)
 {
 	LinebufType		linebuf;
 	RowBucketType	rowbuckets, *rb;
@@ -1319,22 +1372,8 @@ read_and_format(FILE *fp, Options *opts, DataDesc *desc, const char **err)
 
 	memset(desc, 0, sizeof(DataDesc));
 
-	if (!opts->query)
-	{
-		if (fp != NULL)
-		{
-			if (opts->pathname != NULL)
-			{
-				char	   *name;
-
-				name = basename(opts->pathname);
-				strncpy(desc->filename, name, 64);
-				desc->filename[64] = '\0';
-			}
-		}
-		else
-			fp = stdin;
-	}
+	state->errstr = NULL;
+	state->_errno = 0;
 
 	desc->title[0] = '\0';
 	desc->title_rows = 0;
@@ -1377,7 +1416,7 @@ read_and_format(FILE *fp, Options *opts, DataDesc *desc, const char **err)
 
 	if (opts->query)
 	{
-		if (!pg_exec_query(opts, &rowbuckets, &pdesc, err))
+		if (!pg_exec_query(opts, &rowbuckets, &pdesc, &state->errstr))
 			return false;
 	}
 	else if (opts->csv_format)
@@ -1386,8 +1425,8 @@ read_and_format(FILE *fp, Options *opts, DataDesc *desc, const char **err)
 				 &linebuf,
 				 opts->csv_separator,
 				 opts->force8bit,
-				 fp, opts->ignore_short_rows,
-				 opts->nullstr);
+				 state->fp, opts->ignore_short_rows,
+				 opts);
 
 		prepare_pdesc(&rowbuckets, &linebuf, &pdesc, &pconfig);
 	}
@@ -1396,9 +1435,9 @@ read_and_format(FILE *fp, Options *opts, DataDesc *desc, const char **err)
 		read_tsv(&rowbuckets,
 				 &linebuf,
 				 opts->force8bit,
-				 fp,
+				 state->fp,
 				 opts->ignore_short_rows,
-				 opts->nullstr);
+				 opts);
 
 		prepare_pdesc(&rowbuckets, &linebuf, &pdesc, &pconfig);
 	}
@@ -1478,9 +1517,9 @@ read_and_format(FILE *fp, Options *opts, DataDesc *desc, const char **err)
 		 * "translate" headline here (generate translated headline).
 		 */
 		desc->columns = linebuf.maxfields;
-		desc->cranges = smalloc(desc->columns * sizeof(CRange), "prepare metadata");
+		desc->cranges = smalloc2(desc->columns * sizeof(CRange), "prepare metadata");
 		memset(desc->cranges, 0, desc->columns * sizeof(CRange));
-		desc->headline_transl = smalloc(desc->maxbytes + 3, "prepare metadata");
+		desc->headline_transl = smalloc2(desc->maxbytes + 3, "prepare metadata");
 
 		ptr = desc->headline_transl;
 
@@ -1496,7 +1535,7 @@ read_and_format(FILE *fp, Options *opts, DataDesc *desc, const char **err)
 		{
 			int		width = linebuf.widths[i];
 
-			desc->cranges[i].name_pos = -1;
+			desc->cranges[i].name_offset = -1;
 			desc->cranges[i].name_size = -1;
 
 			if (i > 0)
@@ -1595,7 +1634,5 @@ read_and_format(FILE *fp, Options *opts, DataDesc *desc, const char **err)
 		rb = nextrb;
 	}
 
-	*err = NULL;
-
 	return true;
 }
