@@ -1,8 +1,8 @@
+#include "st_menu.h"
+
 #include <ctype.h>
-#include "st_curses.h"
 #include "st_panel.h"
 #include <stdlib.h>
-#include <stdbool.h>
 #include <string.h>
 #include <wchar.h>
 
@@ -21,7 +21,6 @@
 
 #endif
 
-#include "st_menu.h"
 
 /*
  * This window is main application window. It is used for taking content
@@ -808,7 +807,7 @@ debug_print_size(WINDOW *window, char *name)
 #endif
 
 /*
- * adjust pulldown possition - move panels from ideal position to any position
+ * adjust pulldown position - move panels from ideal position to any position
  * where can be fully displayed.
  */
 static void
@@ -1558,7 +1557,7 @@ _st_menu_driver(struct ST_MENU *menu, int c, bool alt, MEVENT *mevent,
 	cursor_row = menu->cursor_row;		/* number of active menu item */
 	is_menubar = menu->is_menubar;		/* true, when processed object is menu bar */
 
-	/* Fucus filter */
+	/* Focus filter */
 	if ((menu->focus == ST_MENU_FOCUS_MOUSE_ONLY && c != KEY_MOUSE) ||
 		(menu->focus == ST_MENU_FOCUS_ALT_MOUSE && c != KEY_MOUSE && !alt) ||
 		(menu->focus == ST_MENU_FOCUS_NONE))
@@ -1620,7 +1619,20 @@ _st_menu_driver(struct ST_MENU *menu, int c, bool alt, MEVENT *mevent,
 			return is_top ? true : false;
 		}
 
-		if (c == KEY_MOUSE && mevent->bstate & (BUTTON1_PRESSED | BUTTON1_RELEASED))
+		/*
+		 * Unpost menu (close) when mouse click outside menu window. Mouse position
+		 * event is processed specially. We want to close only submenu, or we want
+		 * to swith to other bar menu item.
+		 */
+		if (c == KEY_MOUSE &&
+			mevent->bstate & (BUTTON1_PRESSED | BUTTON1_RELEASED
+
+#if NCURSES_MOUSE_VERSION > 1
+
+							  | REPORT_MOUSE_POSITION
+
+#endif
+		  ))
 		{
 			int		y = mevent->y;
 			int		x = mevent->x;
@@ -1637,8 +1649,37 @@ _st_menu_driver(struct ST_MENU *menu, int c, bool alt, MEVENT *mevent,
 
 			if (!is_menubar && !wenclose(menu->draw_area, y, x))
 			{
+
+#if NCURSES_MOUSE_VERSION > 1
+
+				if (mevent->bstate & REPORT_MOUSE_POSITION)
+				{
+					/*
+					 * We want to ignore move movement outside menu's window. Without
+					 * it the behaviour is too sensitive, and it is not user friendly.
+					 * But we need to close submenus, and we want to allow usage of
+					 * bar menu. For these case we should to allow sending event to
+					 * upper objects.
+					 */
+					if (is_nested_pulldown)
+						return false;
+					else
+						/* true when we are not in menu bar position */
+						return y != 0;
+				}
+				else
+				{
+					*unpost_submenu = true;
+					return false;
+				}
+
+#else
+
 				*unpost_submenu = true;
 				return false;
+
+#endif
+
 			}
 		}
 
@@ -1669,7 +1710,14 @@ _st_menu_driver(struct ST_MENU *menu, int c, bool alt, MEVENT *mevent,
 
 #endif
 
-		if (mevent->bstate & (BUTTON1_PRESSED | BUTTON1_RELEASED))
+		if (mevent->bstate & (BUTTON1_PRESSED | BUTTON1_RELEASED
+
+#if NCURSES_MOUSE_VERSION > 1
+
+		 | REPORT_MOUSE_POSITION
+
+#endif
+		  ))
 		{
 			if (is_menubar)
 			{
@@ -1857,7 +1905,15 @@ _st_menu_driver(struct ST_MENU *menu, int c, bool alt, MEVENT *mevent,
 				processed = true;
 				post_menu = true;
 
-				if (mevent->bstate & BUTTON1_PRESSED)
+				if (mevent->bstate & BUTTON1_PRESSED
+
+#if NCURSES_MOUSE_VERSION > 1
+
+				|| mevent->bstate & REPORT_MOUSE_POSITION
+
+#endif
+
+				  )
 				{
 					menu->mouse_row = mouse_row;
 				}
@@ -1928,8 +1984,6 @@ _st_menu_driver(struct ST_MENU *menu, int c, bool alt, MEVENT *mevent,
 		}
 	}
 
-
-
 	/* when menubar is changed, unpost active pulldown submenu */
 	if (menu->active_submenu && cursor_row != menu->cursor_row)
 	{
@@ -2696,7 +2750,15 @@ cmdbar_driver(struct ST_CMDBAR *cmdbar, int c, bool alt, MEVENT *mevent)
 	ST_CMDBAR_ITEM *cmdbar_item = cmdbar->cmdbar_items;
 	ST_MENU_CONFIG *config = cmdbar->config;
 
-	if (c == KEY_MOUSE && mevent->bstate & (BUTTON1_PRESSED | BUTTON1_RELEASED))
+	if (c == KEY_MOUSE &&
+		mevent->bstate & (BUTTON1_PRESSED | BUTTON1_RELEASED
+
+#if NCURSES_MOUSE_VERSION > 1
+
+						  | REPORT_MOUSE_POSITION
+
+#endif
+	  ))
 	{
 		int		y = mevent->y;
 		int		x = mevent->x;
