@@ -1,9 +1,10 @@
-#include "st_curses.h"
-
 #ifndef _ST_MENU_H
 
 #define _ST_MENU_H
 
+#include <stdbool.h>
+#include "st_curses.h"
+
 #define ST_MENU_STYLE_MCB			0
 #define ST_MENU_STYLE_MC			1
 #define ST_MENU_STYLE_VISION		2
@@ -25,8 +26,9 @@
 #define ST_MENU_STYLE_TAO			18
 #define ST_MENU_STYLE_XGOLD			19
 #define ST_MENU_STYLE_XGOLD_BLACK	20
+#define ST_MENU_STYLE_FLATWHITE		21
 
-#define ST_MENU_LAST_STYLE			ST_MENU_STYLE_XGOLD_BLACK
+#define ST_MENU_LAST_STYLE			ST_MENU_STYLE_FLATWHITE
 
 #define	ST_MENU_ESCAPE		27
 
@@ -132,8 +134,8 @@ typedef struct
 
 struct ST_CMDBAR;
 
-extern int st_menu_load_style(ST_MENU_CONFIG *config, int style, int start_from_cpn);
-extern int st_menu_load_style_rgb(ST_MENU_CONFIG *config, int style, int start_from_cpn, int *start_from_rgb);
+extern int st_menu_load_style(ST_MENU_CONFIG *config, int style, int start_from_cpn, bool force8bit, bool force_ascii_art);
+extern int st_menu_load_style_rgb(ST_MENU_CONFIG *config, int style, int start_from_cpn, int *start_from_rgb, bool force8bit, bool force_ascii_art);
 
 extern void st_menu_set_desktop_window(WINDOW *win);
 extern struct ST_MENU *st_menu_new(ST_MENU_CONFIG *config, ST_MENU_ITEM *items, int begin_y, int begin_x, char *title);
