@@ -3,7 +3,7 @@
  * table.c
  *	  a routines for parsing file with data in tabular form
  *
- * Portions Copyright (c) 2017-2020 Pavel Stehule
+ * Portions Copyright (c) 2017-2021 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/table.c
@@ -100,6 +100,14 @@ isHeadLeftChar(char *str)
 	if (str[0] == '-' && str[1] == '[')
 		return true;
 
+	/* csv double header */
+	if ((str[0] == ':' || str[0] == '=') && str[1] == '=')
+		return true;
+
+	/* monetdb */
+	if (str[0] == '+' && str[1] == '=')
+		return true;
+
 	if (strncmp(str, u1, 3) == 0)
 		return true;
 	if (strncmp(str, u2, 3) == 0)
@@ -235,6 +243,16 @@ is_cmdtag(char *str)
 	}
 	else if (*str == 'C')
 	{
+		if (strncmp(str, "CREATE TABLE", 7) == 0)
+		{
+			/*
+			 * MonetDB client show table definition in DDL format. Unfortunately
+			 * it is on first line, so there is partial matching with status
+			 * line format. But Postgres status line is exactly "CREATE TABLE"
+			 * string. It is not hard to different these two situation.
+			 */
+			return str[7] == '\0';
+		}
 		if (strcmp(str, "CALL") == 0 ||
 				strcmp(str, "CHECKPOINT") == 0 ||
 				strncmp(str, "CLOSE", 5) == 0 ||
@@ -737,6 +755,9 @@ next_row:
 
 	if (desc->border_head_row != -1)
 	{
+		if (desc->border_head_row == 0 && !desc->is_expanded_mode)
+			goto broken_format;
+
 		desc->headline = desc->rows.rows[desc->border_head_row];
 		desc->headline_size = strlen(desc->headline);
 
@@ -754,6 +775,8 @@ next_row:
 	}
 	else
 	{
+broken_format:
+
 		desc->headline = NULL;
 		desc->headline_size = 0;
 		desc->headline_char_size = 0;
@@ -940,7 +963,7 @@ translate_headline(Options *opts, DataDesc *desc)
 			*destptr++ = 'd';
 			srcptr += 3;
 		}
-		else if (*srcptr == '+')
+		else if (*srcptr == '+' || *srcptr == ':')
 		{
 			if (processed_chars == 0)
 			{
@@ -958,11 +981,11 @@ translate_headline(Options *opts, DataDesc *desc)
 				if (desc->border_type == 0)
 					desc->border_type = 1;
 
-				*destptr++ = (srcptr[1] == '-') ? 'I' : 'R';
+				*destptr++ = (srcptr[1] == '-' || srcptr[1] == '=') ? 'I' : 'R';
 			}
 			srcptr += 1;
 		}
-		else if (*srcptr == '-')
+		else if (*srcptr == '-' || *srcptr == '=')
 		{
 			if (processed_chars == 0)
 			{
@@ -1033,7 +1056,7 @@ translate_headline(Options *opts, DataDesc *desc)
 		{
 			last_black_char[0] = 'd';
 			last_black_char[1] = 'R';
-			last_black_char[3] = '\0';
+			last_black_char[2] = '\0';
 		}
 
 		/* trim ending spaces */
@@ -1155,10 +1178,15 @@ translate_headline(Options *opts, DataDesc *desc)
 }
 
 /*
- * Cut text from column and translate it to number.
+ * Cut text from column.
  */
 static bool
-cut_text(char *str, int xmin, int xmax, bool border0, bool force8bit, char **result)
+cut_text(char *str,
+		 int xmin,
+		 int xmax,
+		 bool border0,
+		 bool force8bit,
+		 char **result)
 {
 #define TEXT_STACK_BUFFER_SIZE		1024
 
