@@ -3,7 +3,7 @@
  * themes.c
  *	  themes initialization
  *
- * Portions Copyright (c) 2017-2020 Pavel Stehule
+ * Portions Copyright (c) 2017-2021 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/themes.c
@@ -110,11 +110,10 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(21, -1, -1, false, 0);							/* rownum colors */
 			set_colour(22, -1, -1, false, A_REVERSE);					/* cross cursor data */
 			set_colour(23, -1, -1, false, A_REVERSE);					/* cross cursor line */
-			set_colour(22, -1, -1, false, A_REVERSE);					/* cross cursor data */
-			set_colour(23, -1, -1, false, A_REVERSE);					/* cross cursor line */
 			set_colour(24, COLOR_BLACK, COLOR_WHITE, false, cursor_attr);/* vertical cursor pattern data */
 			set_colour(25, COLOR_BLACK, COLOR_WHITE, false, cursor_attr);/* vertical cursor pattern line */
 			set_colour(27, -1, -1, false, 0);		/* input */
+			set_colour(28, COLOR_BLACK, COLOR_WHITE, false, 0);			/* color of bookmark lines for line art */
 			break;
 
 		case 1:
@@ -145,7 +144,8 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(23, COLOR_CYAN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
-			set_colour(26, COLOR_WHITE, COLOR_RED, true,0);
+			set_colour(26, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 2:
@@ -177,6 +177,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(26, COLOR_YELLOW, COLOR_RED, true,0);
+			set_colour(28, COLOR_WHITE, COLOR_MAGENTA, true,0);
 			break;
 
 		case 3:
@@ -207,6 +208,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(23, COLOR_WHITE, COLOR_CYAN, true, A_REVERSE);
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE);
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 4:
@@ -237,7 +239,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);		/* input */
-
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 5:
@@ -268,6 +270,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(23, COLOR_WHITE, COLOR_BLACK, false, 0);
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 6:
@@ -298,6 +301,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, 0);		/* input */
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 7:
@@ -329,6 +333,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_CYAN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_CYAN, COLOR_GREEN, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, 0);		/* input */
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 8:
@@ -360,6 +365,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_CYAN, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);		/* input */
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 9:
@@ -390,6 +396,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(23, COLOR_CYAN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 10:
@@ -421,6 +428,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_BLUE, true, A_REVERSE);
 			set_colour(27, COLOR_BLACK, COLOR_CYAN, false, 0);		/* input */
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 11:
@@ -451,6 +459,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_CYAN, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, 0);		/* input */
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 12:
@@ -481,6 +490,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(23, COLOR_CYAN, COLOR_BLACK, false, 0);
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_BLUE, true, A_REVERSE);
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 13:
@@ -511,6 +521,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(27, COLOR_BLACK, COLOR_CYAN, false, 0);		/* input */
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 		case 14:
 			/* DBase retro magenta */
@@ -541,6 +552,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLUE, true, 0);		/* input */
+			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
 			break;
 
 		case 15:
@@ -571,6 +583,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_RED, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_RED, COLOR_BLACK, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);		/* input */
+			set_colour(28, COLOR_YELLOW, COLOR_RED, true, A_REVERSE | A_BOLD);
 			break;
 
 		case 16:
@@ -603,6 +616,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, -1, -1, true, A_REVERSE | cursor_attr);
 			set_colour(25, -1, -1, true, A_REVERSE);
 			set_colour(27, -1, -1, false, 0);		/* input */
+			set_colour(28, -1, -1, false, A_UNDERLINE);
 			break;
 
 		case 17:
@@ -645,6 +659,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			init_pair(23, 235, 137);
 			init_pair(24, 235, 138);
 			init_pair(25, 235, 138);
+			init_pair(28, 230, 160);
 
 			theme_attrs[4] = labels_attr;
 			theme_attrs[5] = cursor_attr;
@@ -690,6 +705,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			init_pair(23, 235, 137);
 			init_pair(24, 235, 138);
 			init_pair(25, 235, 138);
+			init_pair(28, 255, 160);
 
 			theme_attrs[4] = labels_attr;
 			theme_attrs[5] = cursor_attr;
@@ -713,6 +729,8 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			init_color_rgb_ff(212, 0x00, 0x5f, 0x87); /* mark - faded blue */
 			init_color_rgb_ff(213, 0xfb, 0xf1, 0xc7); /* cursor bg */
 			init_color_rgb_ff(214, 0xd0, 0xcf, 0xa0); /* mark line cursor */
+			init_color_rgb_ff(215, 0xff, 0xff, 0xff); /* mark fg - white */
+
 
 			init_pair(1, 204, 200);
 
@@ -727,7 +745,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			init_pair(10, 203, 205);
 			init_pair(11, 206, 205);
 			init_pair(13, 203, 202);
-			init_pair(14, COLOR_WHITE, 211);
+			init_pair(14, 215, 211);
 			init_pair(15, 212, 204);
 			init_pair(16, 201, 204);
 			init_pair(17, 206, 204);
@@ -739,7 +757,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			init_pair(23, 206, 213);
 			init_pair(24, 201, 214);
 			init_pair(25, 206, 214);
-
+			init_pair(28, 215, 211);
 
 			theme_attrs[4] = labels_attr;
 			theme_attrs[5] = cursor_attr;
@@ -791,6 +809,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			init_pair(19, -1, -1);
 			init_pair(20, 212, 205);
 			init_pair(21, 204, 207);
+			init_pair(28, 212, 211);
 
 			set_colour(22, COLOR_WHITE, COLOR_BLACK, true, cursor_attr);
 			set_colour(23, 204, COLOR_BLACK, false, 0);
@@ -804,13 +823,105 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			theme_attrs[6] = cursor_attr;
 			theme_attrs[10] = cursor_attr;
 			break;
+
+		case 21:
+			/* Flatwhite theme */
+			init_color_rgb_ff(200, 0xf7, 0xf3, 0xee); /* background */
+			init_color_rgb_ff(201, 0x60, 0x5A, 0x52); /* foreground */
+			init_color_rgb_ff(202, 0xb9, 0xA9, 0x92); /* modeline bg */
+			init_color_rgb_ff(203, 0x17, 0x17, 0x17); /* modeline fg */
+			init_color_rgb_ff(204, 0xb9, 0xa9, 0x92); /* table decor */
+			init_color_rgb_ff(205, 0x6a, 0x4d, 0xff); /* cursor bg */
+			init_color_rgb_ff(213, 0xf6, 0xf6, 0xf6); /* cursor fg */
+			init_color_rgb_ff(206, 0x9e, 0x9e, 0x9e); /* footer */
+			init_color_rgb_ff(207, 0xf7, 0xf3, 0xee); /* lineno bg */
+
+			init_color_rgb_ff(210, 0x09, 0x09, 0x08); /* labels */
+			init_color_rgb_ff(211, 0Xf7, 0xe0, 0xc3); /* bookmark - bg */
+			init_color_rgb_ff(212, 0x60, 0x5a, 0x52); /* bookmark - fg */
+			init_color_rgb_ff(215, 0xe2, 0xe9, 0xc1); /* mark bg */
+			init_color_rgb_ff(214, 0xe2, 0xe9, 0xc1); /* marked line bg */
+			init_color_rgb_ff(216, 0x25, 0x25, 0x25);
+
+			init_color_rgb_ff(217, 0x40, 0x40, 0x40); /* vertical marked cursor bg */
+			init_color_rgb_ff(218, 0x00, 0x00, 0x00);
+			init_color_rgb_ff(219, 0x5f, 0x45, 0xe5);
+			init_color_rgb_ff(199, 0x87, 0x70, 0xff);
+
+			init_color(240, 40, 50, 200);
+
+			init_pair(1, 204, 200);
+			init_pair(2, 203, 202);
+			init_pair(3, 201, 200);
+			init_pair(4, 210, 200);
+			init_pair(5, 213, 205);
+			init_pair(6, 213, 205);
+			init_pair(7, 203, 202);
+			init_pair(8, 17, 200);
+			init_pair(9, 206, 200);
+			init_pair(10, 213, 205);
+			init_pair(11, 204, 205);
+			init_pair(13, 203, 202);
+			init_pair(14, 212, 211);
+			init_pair(15, 216, 214);
+			init_pair(16, 201, 214);
+			init_pair(17, 204, 214);
+			init_pair(18, -1, -1);
+			init_pair(19, 201, 205);
+			init_pair(20, 218, 205);
+			init_pair(21, 204, 207);
+			init_pair(28, 204, 211);
+
+			set_colour(22, COLOR_WHITE, 199, true, cursor_attr);
+			set_colour(23, 204, 199, false, 0);
+			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);		/* input */
+
+			init_pair(24, 213, 219);
+			init_pair(25, 204, 219);
+
+			theme_attrs[4] = labels_attr;
+			theme_attrs[5] = cursor_attr;
+			theme_attrs[6] = cursor_attr;
+			theme_attrs[10] = cursor_attr;
+			break;
+
+		case 22:
+			/* mc theme */
+			init_pair(1, COLOR_GREEN, COLOR_BLACK);
+
+			set_colour(2, COLOR_RED, COLOR_BLACK, true, 0);
+			set_colour(3, COLOR_CYAN, COLOR_BLACK, false, 0);
+			set_colour(4, COLOR_WHITE, COLOR_BLACK, true, labels_attr);
+			set_colour(5, COLOR_BLACK, COLOR_CYAN, false, cursor_attr);
+			set_colour(6, COLOR_BLACK, COLOR_CYAN, false, 0);
+			set_colour(7, COLOR_RED, COLOR_BLACK, true, 0);
+			set_colour(8, COLOR_RED, COLOR_BLUE, false, 0);
+			set_colour(9, COLOR_YELLOW, COLOR_BLACK, false, 0);
+			set_colour(10, COLOR_BLACK, COLOR_CYAN, false, cursor_attr);
+			set_colour(11, COLOR_GREEN, COLOR_CYAN, true, 0);
+			set_colour(12, COLOR_WHITE, COLOR_CYAN, true, 0);
+			set_colour(13, COLOR_BLACK, COLOR_GREEN, false, 0);
+			set_colour(14, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(15, COLOR_YELLOW, COLOR_GREEN, true, 0);
+			set_colour(16, COLOR_BLACK, COLOR_GREEN, false, 0);
+			set_colour(17, COLOR_GREEN, COLOR_GREEN, true, 0);
+			set_colour(18, COLOR_GREEN, COLOR_BLUE, false, 0);
+			set_colour(19, COLOR_YELLOW, COLOR_CYAN, true, 0);
+			set_colour(20, COLOR_WHITE, COLOR_BLACK, true, 0);
+			set_colour(21, COLOR_WHITE, COLOR_CYAN, true, 0);
+			set_colour(22, COLOR_CYAN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
+			set_colour(23, COLOR_CYAN, COLOR_WHITE, true, A_REVERSE);
+			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
+			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
+			set_colour(26, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(28, COLOR_GREEN, COLOR_RED, true, 0);
+			break;
 	}
 }
 
 #define _in			if_in_int
 #define _notin		if_notin_int
 
-
 void
 initialize_theme(int theme, int window_identifier, bool is_tabular_fmt, bool no_highlight_lines, Theme *t)
 {
@@ -845,7 +956,7 @@ initialize_theme(int theme, int window_identifier, bool is_tabular_fmt, bool no_
 			t->cursor_expi_attr = theme_attr(6);
 			t->cursor_pattern_attr = theme_attr(20);
 			t->bookmark_data_attr = theme_attr(14);
-			t->bookmark_line_attr = theme_attr(14);
+			t->bookmark_line_attr = theme_attr(28);
 			t->cursor_bookmark_attr = theme_attr(14);
 			t->found_str_attr = !no_highlight_lines ? theme_attr(15) : theme_attr(18);
 			t->pattern_data_attr = theme_attr(16);
@@ -871,7 +982,7 @@ initialize_theme(int theme, int window_identifier, bool is_tabular_fmt, bool no_
 			t->cursor_expi_attr = theme_attr(6);
 			t->cursor_pattern_attr = theme_attr(19);
 			t->bookmark_data_attr = theme_attr(14);
-			t->bookmark_line_attr = theme_attr(14);
+			t->bookmark_line_attr = theme_attr(28);
 			t->cursor_bookmark_attr = theme_attr(14);
 			t->found_str_attr = !no_highlight_lines ? theme_attr(15) : theme_attr(18);
 			t->pattern_data_attr = theme_attr(16);
@@ -896,7 +1007,7 @@ initialize_theme(int theme, int window_identifier, bool is_tabular_fmt, bool no_
 			t->cursor_expi_attr = 0;
 			t->cursor_pattern_attr = theme_attr(19);
 			t->bookmark_data_attr = theme_attr(14);
-			t->bookmark_line_attr = theme_attr(14);
+			t->bookmark_line_attr = theme_attr(28);
 			t->cursor_bookmark_attr = theme_attr(14);
 			t->found_str_attr = !no_highlight_lines ? theme_attr(15) : theme_attr(18);
 			t->pattern_data_attr = theme_attr(16);
@@ -919,6 +1030,7 @@ initialize_theme(int theme, int window_identifier, bool is_tabular_fmt, bool no_
 			t->data_attr = theme_attr(21);
 			t->cursor_data_attr = theme_attr(10);
 			t->bookmark_data_attr = theme_attr(14);
+			t->bookmark_line_attr = theme_attr(28);
 			t->cursor_bookmark_attr = theme_attr(14);
 			t->pattern_data_attr = theme_attr(16);
 			t->cursor_bookmark_attr |= A_BOLD | A_REVERSE;
